use crate::{
	board::{Board, PlayerAction},
	util::Symb,
};
use rand::Rng;
use std::num::NonZeroU8;

use super::{MaximizerAgent, MinimizerAgent};

pub struct RandomAgent {}

impl RandomAgent {
	fn random_action(&self, board: &Board) -> PlayerAction {
		let mut rng = rand::thread_rng();
		let n = board.size();

		let pos = rng.gen_range(0..n);
		let symb = match rng.gen_range(0..4) {
			0 => {
				let n = rng.gen_range(0..=9);
				if n == 0 {
					Symb::Zero
				} else {
					Symb::Number(NonZeroU8::new(n).unwrap())
				}
			}
			1 => Symb::Div,
			2 => Symb::Minus,
			3 => Symb::Plus,
			4 => Symb::Times,
			_ => unreachable!(),
		};

		PlayerAction { symb, pos }
	}
}

impl MinimizerAgent for RandomAgent {
	fn step_min(&mut self, board: &Board) -> PlayerAction {
		let mut action = self.random_action(board);
		while !board.can_play(&action) {
			action = self.random_action(board);
		}
		action
	}
}

impl MaximizerAgent for RandomAgent {
	fn step_max(&mut self, board: &Board) -> PlayerAction {
		let mut action = self.random_action(board);
		while !board.can_play(&action) {
			action = self.random_action(board);
		}
		action
	}
}
